
/*
 *  header - $Id: memguard_machdep.h,v 1.2 1997/06/10 18:49:26 qian Exp $
 *
 *  machine dependent definitions
 */


/*********************************************************************
types & constants
*********************************************************************/

/*
 * Types/Consts: MG_SZPAGE
 *               mgVaddr2Paddr
 * Description : information of pages
 */
typedef unsigned long	paddr_t;

/* size of page */
#define MG_SZPAGE	PAGE_SIZE

/* virtual address to physical address */
#define mgVaddr2Paddr(vaddr)	((paddr_t)(vaddr))

/*
 * Types/Consts: __mgEnterCS and __mgLeaveCS
 * Description : enter critical section and leave critical section
 */
/* enter critical section and leave critical section */
/* QQQQ_LINUX: can use XCHG instruction for multiprocessor */
/* busy wait or return immediately and let the same page protection fault
handler try later again */
#define __mgEnterCS()
#define __mgLeaveCS()

/*
 * Types/Consts: __mgCli, __mgSti, __mgTsti
 * Description : disable, enable, and test interrupt
 */
/* disable, enable, and test interrupt */
typedef unsigned long flags_t;
#define __mgCli(flags)				{\
	save_flags(flags);			\
						\
	if (flags & 0x00000200) {		\
        	cli();				\
	}					}
#define __mgSti(flags)				{\
	if (flags & 0x00000200) {		\
        	sti();				\
	}					}
#define __mgTsti(flags)	(flags & 0x00000200)


/*********************************************************************
variables
*********************************************************************/


/*********************************************************************
functions
*********************************************************************/

/* two functions: mgBeforeInit() and mgAfterInit() */
/* machine-dependent work before and after the kernel initialization */

extern u_int mgBeforeInit(u_int memory_start);
extern u_int mgAfterInit(u_int memory_start, u_int memory_end);

/* two functions: mgBeforeSwitch() and mgAfterSwitch() */
/* machine-dependent work before and after the task switch */

extern void mgBeforeSwitch(struct task_struct *prev);
extern void mgAfterSwitch(void);

/* modified from switch_to() in include/asm/system.h */
#define mg_switch_to(prev,next) do { \
__asm__("movl %0,"SYMBOL_NAME_STR(current_set)"\n" \
	: /* no outputs */ \
	: "r" (next)); \
cli(); \
/* set the page protection of the current tss structure to read-write */ \
mgBeforeSwitch(prev); \
__asm__("ljmp %0\n" \
	: /* no outputs */ \
	: "m" (*(((char *)&next->tss.tr)-4))); \
/* set the page protection of the current tss structure to the original one */ \
mgAfterSwitch(); \
sti(); \
__asm__("cmpl %0,"SYMBOL_NAME_STR(last_task_used_math)"\n\t" \
	"jne 1f\n\t" \
	"clts\n" \
	"1:" \
	: /* no outputs */ \
	: "r" (prev)); \
	/* Now maybe reload the debug registers */ \
	if(prev->debugreg[7]){ \
		loaddebug(prev,0); \
		loaddebug(prev,1); \
		loaddebug(prev,2); \
		loaddebug(prev,3); \
		loaddebug(prev,6); \
	} \
} while (0)

/* memguard page protection fault machine dependent function */

/* write in a page */
extern int mgWrite(char *content,
		  int pageno, u_short offset, u_short size);
extern int mgFaultPages;	/* mgFaultPages is vulnerable to interrupts
					because interrupt handlers may write
					to guarded pages */

/* analyze the current instruction, return the writing length */
extern int  mgAnalyze(long eip);
/* change page protection to read-write, set to signal-step mode */
extern void mgSetEnvironment(struct pt_regs *regs, int pageno);
/* restore page protection and signal-step mode,
	if signal-step mode was enabled formerly, return TRUE */
extern int  mgResetEnvironment(struct pt_regs *regs);

#define __mgRecordCounter(counter_low,counter_high) { \
	unsigned long __eax; \
	unsigned long __edx; \
	__asm__ __volatile__("rdtsc" :"=a" (__eax), "=d" (__edx)); \
	counter_low = __eax; \
	counter_high = __edx; \
	}
#define __mgPrintCounter(msg,counter_low,counter_high) \
printk("%s: 64-bit counter = %08lx:%08lx\n", msg, counter_high, counter_low);

